/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#ifndef VIDEO_H
#define VIDEO_H

#include "SDL.h"
#include "SDL_image.h"
#include "sge.h"
#include "global.h"
#include "player.h"
#include "map.h"
#include "SDL_thread.h"


#define VIDEO_MAXOBJ 100
#define VIDEO_MAXLAYER 5

// Video Thread

SDL_Thread *video;

int VIDEO_UpdateMap;
int VIDEO_UpdateAll;

// Types of Objects

enum {
	V_INVISIBLE = 0,
	V_VISIBLE
};

enum {
	V_NONE = 1,
	V_SURFACE,
	V_TEXT,
	V_INPUT				// Input Field
};

// Objects for Display

struct TObject {
	int layer;
	int type;
	int visible;    	// 0= invisble
	union TData {
		SDL_Surface *surface;
		char *text;
	} data;
	int maxchars;		// only used for input fields: maxchars shown
	SDL_Rect pos;     	// Position
};

// Array of Pointers to my Struct
struct TObject *obj[VIDEO_MAXOBJ];

// Text Display: Fonts

sge_TTFont *font;


//
// Function prototypes
//
// Initialize and thread functions
int VIDEO_InitSDL ();							// Initialize SDL and Video stuff
int VIDEO_Thread ();							// Video-Output Thread
int VIDEO_StartThread ();						// start the thread
void VIDEO_StopThread ();						// stop the thread

// TObject functions
void VIDEO_DrawAll ();							// draw all Tobjects in list
int VIDEO_DrawObject (int index);				// draw a single object
struct TObject * VIDEO_CreateObject (); 		// create a TObject
int VIDEO_RegisterObject (struct TObject *o);	// register a TObject in list
void VIDEO_UnRegisterObject (int index);		// unregister object (frees also)
void VIDEO_FreeObject (struct TObject *o);		// Free a object memory
	
struct TObject *VIDEO_CreateSurfaceObject 		// create a surface object by parameters
	( int layer, int visible, SDL_Surface *surface, SDL_Rect *pos) ;
struct TObject * VIDEO_CreateTextObject 		// create a text out object
	( int layer, int visible, char *text, SDL_Rect *pos);
int VIDEO_TextOut (char *text, int x, int y);	// shortcut to output a text
int VIDEO_InputField 							// shortcut to output a input field (DO NOT USE)
	(char *txt, int txtlen, int x, int y);
void VIDEO_HideObject (int index);				// make a object invisible
void VIDEO_ShowObject (int index);				// make it visible again
void VIDEO_ShowObjectPos 						// make it visible on a new position
	(int index, int x, int y);
void VIDEO_ChangeText 							// change text and position of a text obj
	(int index, char *new_text, int x, int y);

// basic Event functions, timers etc.
int VIDEO_ShowMonthYear (char *str);

#endif
