/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Defender 
 * 
 * initial Version 2003-07-27 Dubspeed
 * release V0.1 2003-08-06
 * release V0.2
 */

#include <stdio.h>
#include "SDL.h"
#include "SDL_image.h"
#include "sge.h"
#include "global.h" 
#include "player.h"
#include "map.h"
#include "menu.h"

/*
   global functions, tools, random number generators ...
   event handling
*/

//
// Timer callback
//
// will we called when the GAMESPEED timner is up
// fires user-events when nessessary
static Uint32 TIMER_Callback(Uint32 interval, void *param) {
	static char str[30];
	SDL_UserEvent e;
	
	// event 1: month change

	global.monat += 1;
	if (global.monat == 13) { global.jahr += 1; global.monat = 1; }
	switch (global.monat) {
		case 1: sprintf (str, JAN); break;
		case 2: sprintf (str, FEB); break;
		case 3: sprintf (str, MRZ); break; 
		case 4: sprintf (str, APR); break;
		case 5: sprintf (str, MAI); break;
		case 6: sprintf (str, JUN); break;
		case 7: sprintf (str, JUL); break;
		case 8: sprintf (str, AUG); break;
		case 9: sprintf (str, SEP); break;
		case 10: sprintf (str, OKT); break;
		case 11: sprintf (str, NOV); break;
		case 12: sprintf (str, DEZ); break;
		default: sprintf (str, ""); break;
	}
	sprintf (str, "%s %i", str, global.jahr);	
	
	e.type = SDL_USEREVENT;
	e.code = EVENT_NEW_MONTH;
	e.data1 = str;
	e.data2 = NULL;
	SDL_PushEvent (&e);
	
	// event 2: trees grow
	global.tree_counter++;
	if (global.tree_counter == GLOBAL_TREEGROWTHRATE) {
		global.tree_counter = 0;
		e.type = SDL_USEREVENT;
		e.code = EVENT_TREES_GROW;
		e.data1 = NULL;
		e.data2 = NULL;
		SDL_PushEvent (&e);
	}

	// restart timer 

	return interval;
} // timer_callback


// SDL support functions, from SDL homepage www.libsdl.org
/*
 * Return the pixel value at (x, y)
 * NOTE: The surface must be locked before calling this!
 */
Uint32 TOOLS_getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;
    case 2:
        return *(Uint16 *)p;
    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;
    case 4:
        return *(Uint32 *)p;
    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
} // TOOLS_getpixel

//
// Random number initialization
// <- 0 = success
//  
// does not work on WIN-Machines : alternitives ?
unsigned int TOOLS_InitRandom () {
	FILE *dat;
	
	dat = fopen("/dev/urandom", "r");
	if (dat == NULL) return -1;
	fread(random, sizeof(random[0]), 65536, dat); 
	fclose(dat);

	random_c = 0;
	return 0;
} // Tools_InitRandom

// 
// Random number
// <- random number between 0..255
unsigned char TOOLS_Random () {
	if (random_c == 4096) random_c = 0;
	return random[random_c++];
} // Tools_random


/* 
//
//   M A I N   M A I N   M A I N
//
*/

int main(int argc, char *argv[])
{
	SDL_Event event;
	SDL_TimerID timer;
 
	if ( VIDEO_InitSDL () < 0 ) {
        fprintf(stderr, "SDL could not be initialized: %s\n", SDL_GetError());
        exit(1);
    }
    
	PLAYER_InitPlayer ();
	
	if (TOOLS_InitRandom () < 0) {
		fprintf(stderr, "Could not initialize random number generator: %s\n", SDL_GetError());
        exit(1);
	}
	
	if (MAP_InitMap() < 0) {
        fprintf(stderr, "Could not initialize the map: %s\n", SDL_GetError());
        exit(1);		
	}
	
	global.jahr = 915;
	global.monat = 1;	
	global.tree_counter = 0;

	// Lade die Mens
	MENU_Load ();
	
	// Timer 
	timer = SDL_AddTimer(GAMESPEED, TIMER_Callback, NULL);
	
	// Threads
	if (VIDEO_StartThread () < 0) {
		fprintf(stderr, "Could not start the video thread: %s\n", SDL_GetError());
        exit(1);
	}

	// start top left
	MAP_posx = 0;
	MAP_posy = 0;

	// build up the map
	MAP_ShowMap (MAP_posx, MAP_posy);
	
	// 
	VIDEO_TextOut(player[PLAYER_HUMAN].name, 400, 25);

	// Main loop
	while (1 != 0) 
//	while (SDL_PollEvent (&event) == 1)  // dont wait, check and run
	while (SDL_WaitEvent (&event) == 1)  // wait for an event
	{
//		lframe = SDL_GetTicks ();
		switch (event.type)
		{
		case SDL_QUIT:	
     		VIDEO_StopThread ();
			exit(0);
			break;
		
		case SDL_MOUSEBUTTONDOWN:
			MAPED_MouseButton (&event.button);
			break;
		
		case SDL_MOUSEBUTTONUP:
			MAPED_MouseButtonUp (&event.button);
			break;
		
		case SDL_MOUSEMOTION:
			MAPED_MouseMotion (&event.motion);
			break;
		
		case SDL_KEYDOWN:
			switch (event.key.keysym.sym)
			{
			case SDLK_ESCAPE:
				exit (0);
				break;
			
			case SDLK_LEFT:
			    MAP_MoveLeft (1);
				VIDEO_UpdateMap = TRUE;
				break;
			case SDLK_RIGHT:
			    MAP_MoveRight (1);
				VIDEO_UpdateMap = TRUE;
				break;
			case SDLK_UP:
			    MAP_MoveUp (1);
				VIDEO_UpdateMap = TRUE;
				break;
			case SDLK_DOWN:
			    MAP_MoveDown(1);
				VIDEO_UpdateMap = TRUE;
				break;
			case SDLK_F11:			// Save map
				MAP_SaveMap();
				break;
			case SDLK_F12:			// Load map
				MAP_LoadMap();
				break;			
			default:
				break;
			}
			break;
		case SDL_USEREVENT:
			switch (event.user.code) 
			{
				case EVENT_NEW_MONTH: 
					VIDEO_ShowMonthYear (event.user.data1);
					break;
				case EVENT_TREES_GROW:
					MAP_GrowTrees ();
					VIDEO_UpdateMap = TRUE;
					break;
			}

		break;
		}		// switch event.type
	
		SDL_Delay (1);
		
//		while ((SDL_GetTicks() - lframe) < 100) ;	// another type of waiting instead of delay
	}			// MAIN LOOP

	
	return (0);

}
