/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "include/map.h"


int MAP_InitMap () {
	
	int i;
	unsigned char zuf;
	int index;
	
	for (i=0; i<MAP_X*MAP_Y; i++) {
		map[i].grnd_img = NULL;
		map[i].build_img = NULL;
		map[i].grnd_nr = 0;
		map[i].build_nr = 0;
	}
	
	if (MAP_LoadImages(MAP_DESCR_FILE) != 0) return -1;


	
	// Initialize Map
	// put random gras on the map
	for (i=0; i<MAP_X*MAP_Y; i++) {
		zuf = TOOLS_Random();
		//printf ("%i\n", zuf);
		if (zuf <= 64) {  
			index = MAP_GetIndexByType (MT_GRAS_1);
			map[i].grnd_img = tiles[index].img;
			map[i].grnd_nr = tiles[index].index;
			map[i].grnd_tile = &tiles[index];
		} else if (zuf >64 && zuf <= 100) {
			index = MAP_GetIndexByType (MT_GRAS_2);
			map[i].grnd_img = tiles[index].img;
			map[i].grnd_nr = tiles[index].index;
			map[i].grnd_tile = &tiles[index];
		} else if (zuf >100 && zuf <=200) {
			index = MAP_GetIndexByType (MT_GRAS_3);
			map[i].grnd_img = tiles[index].img;
			map[i].grnd_nr = tiles[index].index;
			map[i].grnd_tile = &tiles[index];
		} else if (zuf >200) {
			index = MAP_GetIndexByType (MT_GRAS_4);
			map[i].grnd_img = tiles[index].img;
			map[i].grnd_nr = tiles[index].index;
			map[i].grnd_tile = &tiles[index];
		} //if
	}  // for 
	
	
	// put some trees on it
	// -> target is to group the trees
	
	for (i=0; i<MAP_X*MAP_Y; i++) {
		zuf = TOOLS_Random();
		//printf ("%i\n", zuf);
		if (zuf <= 32) { 
			index = MAP_GetIndexByType (MT_TREE_1);
			map[i].build_img = tiles[index].img;
			map[i].build_nr = tiles[index].index;
			map[i].build_tile = &tiles[index];		
		} else if (zuf >32 && zuf <= 50) {
			index = MAP_GetIndexByType (MT_TREE_2);
			map[i].build_img = tiles[index].img;
			map[i].build_nr = tiles[index].index;
			map[i].build_tile = &tiles[index];	
		} else if (zuf >50 && zuf <=64) {
			index = MAP_GetIndexByType (MT_TREE_3);
			map[i].build_img = tiles[index].img;
			map[i].build_nr = tiles[index].index;
			map[i].build_tile = &tiles[index];	
		}	// if	
	} // for

	return 0;
}

int MAP_MoveLeft (int steps) {
	if ((MAP_posx - steps) <= 0) return -1;
	MAP_posx = MAP_posx - steps;
	return 0;
}

int MAP_MoveRight (int steps) {
	if ((MAP_posx + steps) >= MAP_X - MAP_WEI) return -1;
	MAP_posx = MAP_posx + steps;
	return 0;
}

int MAP_MoveUp (int steps) {
	if ((MAP_posy - steps) <= 0) return -1;
	MAP_posy = MAP_posy - steps;
	return 0;
}

int MAP_MoveDown (int steps) {
	if ((MAP_posy + steps) >= MAP_Y - MAP_HEI) return -1;
	MAP_posy = MAP_posy + steps;	
	return 0;
}

//
//   Zeige die Karte an, beginne in der linken oberen Ecke des
//   Screens und immer 40 Pixel pro Feld 
//   definiert in MAP_H und MAP_W
//   x, y = Beziehen sich immer auf das Tile nicht auf die Pixel-position
//   
// 	 V0.3 Beta
//
//   TODOs:
//   Scollen an den oberen Rand und an den Rechten Rand fhrt zu wilden "Sprngen"
//   -> wenn ungltige Werte gefunden werden springt er zu 0,0  ! 
//   -> es scheint also ungltige positionen zu geben ! TODO: das unterbinden
int MAP_ShowMap (unsigned int startx, unsigned int starty) {
	int x;
	SDL_Rect r_scr;
	
	if (startx > MAP_X - MAP_WEI) { 
		startx = MAP_X - MAP_WEI;
		return -1;	
	}

	if (starty > MAP_Y - MAP_HEI) {
		starty = MAP_Y - MAP_HEI;
		return -1;	
	}

	// Anzeigestart auf Screen
	// Sollte evtl. noch angepasst werden da nicht der ganze Screen zu sehen ist
	r_scr.x = MAP_STARTPOINT_X; 
	r_scr.y = MAP_STARTPOINT_Y;
	
	// Berechne Starttile im Speicherarray
	// X= Zhler ber das Speicherarray
	// +1 um das Offset der ersten Zeile richtig zu setzen
	x = starty * MAP_X + startx + 1;
	
	while (r_scr.y < MAP_ENDPOINT_Y) {
		SDL_BlitSurface (map[x].grnd_img, NULL, scr, &r_scr);
		if (map[x].build_nr != 0) 
			SDL_BlitSurface (map[x].build_img, NULL, scr, &r_scr);
		r_scr.x += TILE_X;
		if (r_scr.x == MAP_ENDPOINT_X - TILE_X) {
		  	r_scr.y += TILE_Y;
		  	r_scr.x = 0;
			// aber auch neue Speicherposition errechnen
		  	starty++;
		 	x = starty * MAP_X + startx;
		} // if
		x++;		
	} // while x
	return 0;
 
}  // ShowMap



// Save MAP into a file
int MAP_SaveMap (void) {
	FILE *dat;
	int i,j;
	char *p_map; 
	int txt;
	char filename[30] = "default.map";

    txt = VIDEO_TextOut("Filename:", 400, 25);
	VIDEO_UpdateAll = TRUE;

	sge_tt_input (scr, font, filename,
	                 SGE_IBG | SGE_IDEL, 0, 30, 
					 500, 25,
 	                 0x9e9e9e, 
					 0xd38d15,
	                 255);

	VIDEO_UpdateAll = TRUE;
	VIDEO_UnRegisterObject (txt);

	printf ("saving file %s\n", filename);

	txt = VIDEO_TextOut ("saving....", 500, 25); 
    VIDEO_UpdateAll = TRUE;

	dat = fopen (filename, "wb");
	
	// Loop over map array and write the objects byte by byte
	
	for (i=0; i<MAP_X * MAP_Y; i++) // for each map object
	{
		p_map = (char *) &map[i];
		for (j=0; j<sizeof(map[i]); j++) {
			fputc (*p_map, dat);
			p_map++;
		}
	}
	
	fclose (dat);
	VIDEO_UnRegisterObject (txt);
    VIDEO_UpdateAll = TRUE;
	return 0;
} // MAP_SaveMap


// Lade Informationen in die MAP
// nach dem Laden stehen die Zeiger auf die Grafiken auf den "alten" Werten
// aus dem File und mssen neu zugewiesen werden !
// 
// strzt manchmal reproduzierbar ab und reisst alles mit sich !
// -> vielleicht Video-Thread dazu unterbinden ?
// Korrektur eingefgt: besser jetzt ???

int MAP_LoadMap (void) {
	FILE *dat;
	int i;
	char *p_map;
	int txt;
	char filename[30] = "default.map";

    txt = VIDEO_TextOut("Filename:", 400, 25);
	VIDEO_UpdateAll = TRUE;

	sge_tt_input (scr, font, filename,
	                 SGE_IBG | SGE_IDEL, 0, 30, 
					 500, 25,
 	                 0x9e9e9e, 
					 0xd38d15,
	                 255);

	VIDEO_UpdateAll = TRUE;
	VIDEO_UnRegisterObject (txt);

	printf ("loading file %s\n", filename);

	txt = VIDEO_TextOut ("loading....", 500, 25); 
    VIDEO_UpdateAll = TRUE;

	dat = fopen (filename, "rb");
    if (dat == NULL) {
		VIDEO_UnRegisterObject (txt);
		VIDEO_UpdateAll = TRUE;
		return -1;
	}

	p_map = (char *) &map[0];
	for (i=0; i<MAP_X * MAP_Y * sizeof(struct TMap); i++) { 
		*p_map = fgetc (dat); 
		p_map++; 
	}

	fclose (dat);

	// Da sich die Zeiger auf die Grafiken verndert haben mssen
	// diese neu in die Objekte "verlinkt" werden.
	
	for (i=0; i<MAP_X * MAP_Y; i++) {
			map[i].grnd_img = tiles[map[i].grnd_nr].img;
			map[i].grnd_tile = &tiles[map[i].grnd_nr];		
			map[i].build_img = tiles[map[i].build_nr].img;
			map[i].build_tile = &tiles[map[i].build_nr];		
	} // for 
	
	VIDEO_UnRegisterObject (txt);
	VIDEO_UpdateAll = TRUE;
	VIDEO_UpdateMap = TRUE;

	return 0;

} // MAP_LoadMap


//
//
// Scrollonmouse:
// Scrolle wenn die Mouseposition den rand erreicht
//
void MAP_ScrollonMouse (SDL_MouseMotionEvent *e) {
 
	if (e->x < 50) { MAP_posx--; return ; }
	if (e->x > SCR_X - 50) { MAP_posx++; return ; }	  
	if (e->y < 50) { MAP_posy--; return ; }
    if (e->y > SCR_Y - 50) { MAP_posy++; return ; }	 
}

// errechne das richtige Tile aus der aktuellen Cursorposition und gebe die Tilenummer zurck
// = index in der MAP
// -> x -Position als Cursorposition
// -> y -Position
// <- index im MAP Array
int MAP_GetTileforPos (int x, int y) {
	/*
		X-Position - X-Startwert auf dem Schirm
		Rest durch Tilebreite teilen = Tile
		
	*/
	int pos_x, pos_y;

	// check 1: ist die Position auf der MAP ?
	if (x < MAP_STARTPOINT_X ) return -1;
	if (x > MAP_ENDPOINT_X) return -1;
	if (y < MAP_STARTPOINT_Y) return -1;
	if (y > MAP_ENDPOINT_Y) return -1;

	// rechne: welches tile auf dem schirm?    
	pos_x = (x - MAP_STARTPOINT_X) / TILE_X;
	pos_y = (y - MAP_STARTPOINT_Y) / TILE_Y;
	pos_x = pos_x + MAP_posx;
	pos_y = pos_y + MAP_posy;
	
//	printf ("x: %i y: %i\n", x, y);
//	printf ("pos_x: %i pos_y: %i\n", pos_x, pos_y);

	return (pos_y * MAP_X) + pos_x + 1;

}

// Ein Kartenobjekt fr eine Index-position zurckliefern
struct TMap *MAP_GetMapForIndex (int map_index) {
 	return &map[map_index];
}

// Verndere ein Tile in der Map ber seinen Index
// -> Index in der MAP 
// -> Index des Tiles aus dem Tile-Array
// -> Lsche ein bestehendes Buildig wenn Groundtyp gesetzt wird ? 
// <-
void MAP_SetTileIndex (int map_index, int tile_index, char delete_building) {
  
	if (tiles[tile_index].isbuild == FALSE) {
		map[map_index].grnd_nr = tile_index;
		map[map_index].grnd_img = tiles[tile_index].img;
		map[map_index].grnd_tile = &tiles[tile_index];
		if (delete_building == TRUE) {
			map[map_index].build_nr = 0;
			map[map_index].build_img = NULL;
			map[map_index].build_tile = NULL;
		}
	} else {
		map[map_index].build_nr = tile_index;
		map[map_index].build_img = tiles[tile_index].img;
		map[map_index].build_tile = &tiles[tile_index];	
	} // if

}

// Lade die Tiles in den Tilebuffer
// Dateibeschreibung
// Klasse (siehe MAP_CLASS) Leerzeichen ISbuild (1 = ja) Leerzeichen Pfad
// 

int MAP_LoadImages (char *descr_file) {

    SDL_Surface *temp;
	FILE *dat;
	int  tmp_anim_group, tmp_isbuild;
	char tmp_pfad[50], tmp_type[50];
	int index;	
	int i;

	index = 0;

   	// Initialisiere das Array
    for (i=0; i<MAP_MAXTILES; i++) { tiles[i].type = 0; tiles[i].index = 0; }

    // lese das descr_file und erstelle beschreibende Objekte dazu
	dat = fopen (descr_file, "r");
	while ( EOF != fscanf(dat, "%i %i %s %s", &tmp_anim_group, &tmp_isbuild, tmp_pfad, tmp_type) ) {
		printf ("%i %i %s %s\n", tmp_anim_group, tmp_isbuild, tmp_pfad, tmp_type);
		tiles[index].anim_group = tmp_anim_group;
//		tiles[index].nr = index;
		tiles[index].isbuild = tmp_isbuild;
//		tiles[index].type = tmp_type;
		if (strcmp(tmp_type, "GRAS_1") == 0) 
			tiles[index].type = MT_GRAS_1;
		if (strcmp(tmp_type, "GRAS_2") == 0) 
			tiles[index].type = MT_GRAS_2;
		if (strcmp(tmp_type, "GRAS_3") == 0) 
			tiles[index].type = MT_GRAS_3;
		if (strcmp(tmp_type, "GRAS_4") == 0) 
			tiles[index].type = MT_GRAS_4;
		if (strcmp(tmp_type, "TREE_1") == 0) 
			tiles[index].type = MT_TREE_1;
		if (strcmp(tmp_type, "TREE_2") == 0) 
			tiles[index].type = MT_TREE_2;
		if (strcmp(tmp_type, "TREE_3") == 0) 
			tiles[index].type = MT_TREE_3;
		if (strcmp(tmp_type, "LUMBERJACK") == 0) 
			tiles[index].type = MT_LUMBERJACK;
		if (strcmp(tmp_type, "HUNTER") == 0) 
			tiles[index].type = MT_HUNTER;
		if (strcmp(tmp_type, "HOUSE_01") == 0) 
			tiles[index].type = MT_HOUSE_01;
		
		if (strcmp(tmp_type, "WAY_0") == 0) 
			tiles[index].type = MT_WAY_00;
		if (strcmp(tmp_type, "WAY_1") == 0) 
			tiles[index].type = MT_WAY_01;
		if (strcmp(tmp_type, "WAY_2") == 0) 
			tiles[index].type = MT_WAY_02;
		if (strcmp(tmp_type, "WAY_3") == 0) 
			tiles[index].type = MT_WAY_03;
		if (strcmp(tmp_type, "WAY_4") == 0) 
			tiles[index].type = MT_WAY_04;
		if (strcmp(tmp_type, "WAY_5") == 0) 
			tiles[index].type = MT_WAY_05;
		if (strcmp(tmp_type, "WAY_6") == 0) 
			tiles[index].type = MT_WAY_06;
		if (strcmp(tmp_type, "WAY_7") == 0) 
			tiles[index].type = MT_WAY_07;
		if (strcmp(tmp_type, "WAY_8") == 0) 
			tiles[index].type = MT_WAY_08;
		if (strcmp(tmp_type, "WAY_9") == 0) 
			tiles[index].type = MT_WAY_09;
		if (strcmp(tmp_type, "WAY_10") == 0) 
			tiles[index].type = MT_WAY_10;
		if (strcmp(tmp_type, "WAY_11") == 0) 
			tiles[index].type = MT_WAY_11;
		if (strcmp(tmp_type, "WAY_12") == 0) 
			tiles[index].type = MT_WAY_12;
		if (strcmp(tmp_type, "WAY_13") == 0) 
			tiles[index].type = MT_WAY_13;
		if (strcmp(tmp_type, "WAY_14") == 0) 
			tiles[index].type = MT_WAY_14;
		if (strcmp(tmp_type, "WAY_15") == 0) 
			tiles[index].type = MT_WAY_15;

		tiles[index].index = index;	
	
		if ( (temp = IMG_Load (tmp_pfad)) < 0) {
			fprintf (stderr, "Error reading Tile-File: %s\n", tmp_pfad);
			return -1;
		}
		
		tiles[index].img = SDL_DisplayFormat (temp);
		if (tiles[index].isbuild == TRUE) // wenn es ein building ist transparenz setzen
			SDL_SetColorKey (tiles[index].img, SDL_SRCCOLORKEY, SDL_MapRGB (tiles[index].img->format, 255, 0, 255));
		index++;
	}	

	fclose (dat);
    
	return 0;
}


// GetIndexByType
int MAP_GetIndexByType (int type) {
	int i;

	for (i=0; i<MAP_MAXTILES; i++) {
		if (tiles[i].type == type) break;
	}
	
	return i;
}

// Whle einen beliebigen Brush zu einem Brushtyp
struct TTile *MAP_GetTileByType (int type) {

	int i;	

	for (i=0; i<MAP_MAXTILES; i++) {
		if (tiles[i].type == type) break;
	}
	
	return &tiles[i];
}  // GetTileByType


// Ein Tile "nrdlich" des gegebenen Tiles finden
// -> Index des Tiles
// -> Index des angrenzenden Tiles, oder -1 bei Fehler
int MAP_GetMapTileNorth (int index) {
 	if ((index - MAP_X) < 0) return -1;
	return (index - MAP_X);
}

int MAP_GetMapTileSouth (int index) {
 	if ((index + MAP_X) > MAP_X * MAP_Y) return -1;
	return (index + MAP_X);
}

int MAP_GetMapTileEast (int index) {
 	if ((index + 1 ) > MAP_X * MAP_Y) return -1;
	return (index + 1);
}


int MAP_GetMapTileWest (int index) {
 	if ((index - 1 ) < 0) return -1;
	return (index - 1);
}

SDL_Surface *MAP_GetTileImage (int index) {
	return tiles[index].img;
}


// Umgebung (n,S,O,W) eines Tiles scannen und richtigen "Way" vorschlagen
// abhngig von den umliegenden Wegen
// -> index des Tiles, das seine Umgebung "verpfaden" soll
// <- Zeiger auf das zu setzende Tile

struct TTile *MAP_CreatePath (int index) {
	int north, south, east, west;
	int wayval;
	struct TTile *brush;
	static int level = 0;		// recursive control level
	
	wayval = 0;

	north = MAP_GetMapTileNorth (index);
	south = MAP_GetMapTileSouth (index);
	east = MAP_GetMapTileEast (index);
    west = MAP_GetMapTileWest (index);

	//printf ("Levl : %i Index:%i   n%i e%i s%i w%i\n", level, index,north, east, south, west);

	if ((north != -1) 
		&& (map[north].build_tile != NULL) 
		&& (map[north].build_tile->type >= MT_WAY_00) 
		&& (map[north].build_tile->type <= MT_WAY_15)) {
			wayval += 1;
			if (level == 0) { 
				level++; 										// set level value to 1
				brush = MAP_CreatePath (north);  				// recusrse 1 level down
				MAP_SetTileIndex (north, brush->index, TRUE); 
				level = 0;										// we are back, reset level value to 0
			} // if level
		} // if north
	
	if ((east != -1) 
		&& (map[east].build_tile != NULL) 
		&& (map[east].build_tile->type >= MT_WAY_00) 
		&& (map[east].build_tile->type <= MT_WAY_15)) {
			wayval += 2;
			if (level == 0) {
				level++;
				brush = MAP_CreatePath (east);  
				MAP_SetTileIndex (east, brush->index, TRUE); 
				level = 0;
			} // if level
		} // if east
		
	if ((south != -1) 
		&& (map[south].build_tile != NULL) 
		&& (map[south].build_tile->type >= MT_WAY_00) 
		&& (map[south].build_tile->type <= MT_WAY_15)) {
			wayval += 4;
			if (level == 0) { 
				level++; 
				brush = MAP_CreatePath (south);  
				MAP_SetTileIndex (south, brush->index, TRUE); 
				level = 0;
			} // if level
		} // if south
	
	if ((west != -1) 
		&& (map[west].build_tile != NULL) 
		&& (map[west].build_tile->type >= MT_WAY_00) 
		&& (map[west].build_tile->type <= MT_WAY_15)) {
			wayval += 8;
			if (level == 0) { 
				level++; 
				brush = MAP_CreatePath (west);  
				MAP_SetTileIndex (west, brush->index, TRUE); 
				level = 0;
			} // if level
		} // if west
	
	//printf ("mtway_00:%i  val:%i\n", MT_WAY_00, wayval);
	
	return MAP_GetTileByType(MT_WAY_00 + wayval);	
} // MAP_CreatePath


// grow tree by tree on the map
// by displaing the next tile in the same tile.anim_group
int MAP_GrowTrees () {

	int i;				// counter
	unsigned char r;	// random nr
	int neigh;  		// a tile north, east, south or west of ours 

	// loop over map, if it is visible and a vaild tree on it then grow this tree 

	for (i=0; i< MAP_X * MAP_Y; i++) {
		if ( //(map[i].visible == TRUE) &&
			(map[i].build_nr != 0) &&
			(map[i].build_tile->type >= MT_TREE_1) &&
			(map[i].build_tile->type <= MT_TREE_3)) {
			
			// is there a anim_group != 0 for the tree
			// and is the next tile in sequence in the same anim group
			// then set this tile as actual tile for this map-field
			if ((map[i].build_tile->anim_group != 0) &&
				((map[i].build_tile->index + 1) < MAP_MAXTILES) &&
				(tiles[map[i].build_tile->index +1].anim_group == map[i].build_tile->anim_group)) {
				
				map[i].build_nr++;
				map[i].build_tile = &tiles[map[i].build_nr];
				map[i].build_img = map[i].build_tile->img;
				continue;
			} // if
		
			// if we have a already grown up tree (= the next tile in row has a diffrent anim_group) 
			// we scan the area around the tree and randomliy set a little tree on a free area		
			if ((map[i].build_tile->anim_group != 0) &&
				((map[i].build_tile->index + 1) < MAP_MAXTILES) &&
				(tiles[map[i].build_tile->index +1].anim_group != map[i].build_tile->anim_group)) {
			
				// get a random number
				r = TOOLS_Random ();				
				if (r < 16) {
					neigh = MAP_GetMapTileNorth (i);
				} else if (r >= 16 && r < 32) {
					neigh = MAP_GetMapTileEast (i);
				} else if (r >= 32 && r < 48) {
					neigh = MAP_GetMapTileSouth (i);
				} else if (r >= 48 && r < 64) {
					neigh = MAP_GetMapTileWest (i);
				} else {
					continue;
				} // if r
				
				// find the first tile for my type
				if (map[neigh].build_nr == 00) {
					map[neigh].build_tile = MAP_GetTileByType (map[i].build_tile->type);
					map[neigh].build_nr = map[neigh].build_tile->index;
					map[neigh].build_img = map[neigh].build_tile->img;
				}
			
			} // if
		
		} // if build_nr != 0
	} // for i=0 < MAP_X*MAP_Y

	return 0;

} // MAP_Grow_Trees
