/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include "menu.h"
#include <string.h>

int MENU_Load () {
	SDL_Surface *temp;
	struct TObject *p_obj;
	int i;
	
	// Load Menu backgroundimage
	if ( (temp = IMG_Load (MENU_GFX_BACKGROUND) ) < 0 ) return -1;
	back = SDL_DisplayFormat (temp);	
	
	// clear menu area information 
	for (i=0; i<10; i++) {
		MENU_Area[i].x = 0;
		MENU_Area[i].y = 0;
		MENU_Area[i].w = 0;
		MENU_Area[i].h = 0;
	}

	// Registriere Hintergrund bild fr anzeige
	p_obj = VIDEO_CreateSurfaceObject(1, V_VISIBLE, back, NULL);
	VIDEO_RegisterObject(p_obj);
 
	
	// Setze pink = Transparent 
	SDL_SetColorKey (back, SDL_SRCCOLORKEY, SDL_MapRGB (back->format, 255, 0, 255));
	
	// Lade andere Grafiken 
	if ( (temp = IMG_Load (MENU_GFX_HIGHLIGHT) ) < 0 ) return -1;
	frame = SDL_DisplayFormat (temp);	
	SDL_SetColorKey (frame, SDL_SRCCOLORKEY, SDL_MapRGB (frame->format, 255, 0, 255));
	// Registriere Hintergrund bild fr anzeige
	p_obj = VIDEO_CreateSurfaceObject(3, V_INVISIBLE, frame, NULL);
	frame_index = VIDEO_RegisterObject(p_obj);
	
	txt_ln1 = VIDEO_TextOut (TITLE, 15, 25);
	
//	MAPED_Brush = -1; oboslet
//	MAPED_Brush_Type = -1;
	
	// Initialize Men - Array
	for (i=0; i< MENU_MENULEVEL * MENU_MENUENTRIES; i++) {
		menu[i].level = 0;
		menu[i].pos = 0;
		menu[i].text = NULL;
		menu[i].tile = NULL;
		menu[i].brush = NULL;
		menu[i].visible = FALSE;
	}
	
	// Lade das Men Array aus Datei menu.descr
	if (MENU_LoadMenu (MENU_DESCR_FILE) == -1) {
		fprintf (stderr, "Kann Menbeschreibung menu.descr nicht laden\n");
		exit (-1);
	}

	// Grafiken des Menus aus dem Array anzeigen
	MAPED_ShowMenuButtons (MENU_LEVEL_1);
	MENU_Selected_Index = -1;

	return 0;
}

// Lade die Mens aus der Datei Menu.descr
// Level
// Position
// Text
// Type des Tiles
// Index des Brushes der benutzt werden soll
// TODO: Indexwert in der Datei bercksichtigen - Brush ist noch gleich Tile
int MENU_LoadMenu (char *filename) {
	FILE *dat;
	int tmp_level, tmp_pos, tmp_index, tmp_x, tmp_y;
	char tmp_type[255], tmp_text[255];
	int index, index2;
	int str_counter;

	dat = fopen (filename, "r");
	if (dat == NULL) return -1;
		
	printf ("\nLoading menu.descr.:\n");
	printf ("-------------------\n");
	index = 0; index2 = 0;
	while ( EOF != fscanf(dat, "%i%i%i%i%s%s%i", &tmp_level, &tmp_pos, &tmp_x, &tmp_y, tmp_text, tmp_type, &tmp_index) ) {	

		// level 0 describes the 5 menu sensible spots (buttons)
		// level 0 has to be the first 10 entries
		// the other level describe the filling gfx
		if (tmp_level == MENU_LEVEL_AREA) {
			MENU_Area[index2].x = tmp_pos;
			MENU_Area[index2].y = tmp_x;
			MENU_Area[index2].w = tmp_y;
			MENU_Area[index2].h = tmp_index;
			index2++;
//		} else if () {
//			// ...
		} else if (tmp_level == MENU_LEVEL_1 || tmp_level == MENU_LEVEL_2) {
			if (tmp_level == MENU_LEVEL_1) 
				menu[index].visible = TRUE;
			else
				menu[index].visible = FALSE;
			menu[index].index = index;
			menu[index].level = tmp_level;
			menu[index].pos = tmp_pos;	
			menu[index].x	= tmp_x;
			menu[index].y	= tmp_y;
			menu[index].text = (char *) malloc (strlen(tmp_text));
			strcpy(menu[index].text, tmp_text);
			for (str_counter = 0; str_counter < strlen(menu[index].text); str_counter++) 
				if (menu[index].text[str_counter] == '_') menu[index].text[str_counter] = ' ';
//			printf ("%s\n", tmp_type);
			if (strcmp(tmp_type, "GRAS_1") == 0) 
				menu[index].tile = (struct TTile *) MAP_GetTileByType (MT_GRAS_1);
			if (strcmp(tmp_type, "TREE_2") == 0) 
				menu[index].tile = (struct TTile *) MAP_GetTileByType (MT_TREE_2);
			if (strcmp(tmp_type, "LUMBERJACK") == 0) 
				menu[index].tile = (struct TTile *) MAP_GetTileByType (MT_LUMBERJACK);
			if (strcmp(tmp_type, "WAY_00") == 0) 
				menu[index].tile = (struct TTile *) MAP_GetTileByType (MT_WAY_00);
			if (strcmp(tmp_type, "HUNTER") == 0) 
				menu[index].tile = (struct TTile *) MAP_GetTileByType (MT_HUNTER);
			if (strcmp(tmp_type, "HOUSE_01") == 0) 
				menu[index].tile = (struct TTile *) MAP_GetTileByType (MT_HOUSE_01);
					
			menu[index].brush = menu[index].tile;	
 			printf ("%i %i %s %s %i\n", menu[index].level, menu[index].pos, menu[index].text, tmp_type, tmp_index);			
			index++;
		} // if
	} // while

	fclose (dat);
	printf ("-------------------\n");

	return 0;
}

// Show the menu buttons
void MAPED_ShowMenuButtons (int level) {
	SDL_Rect ziel;
	struct TObject *p_obj;
	int i;
		
	for (i=0; i< MENU_MENULEVEL * MENU_MENUENTRIES; i++)
		if (menu[i].level == level) {
			menu[i].visible = TRUE;
			ziel.x = menu[i].x;
			ziel.y = menu[i].y;
			p_obj = VIDEO_CreateSurfaceObject (2, V_VISIBLE, MAP_GetTileImage (menu[i].tile->index) , &ziel);
			VIDEO_RegisterObject (p_obj);
		}

	// update screen
	VIDEO_UpdateAll = TRUE;

} // ShowDesignButtons


// Handle a Mouse down event
int MAPED_MouseButton (SDL_MouseButtonEvent *e) {		
	switch (e->button) {
		case 1: 					// Linker Button
			MAPED_LeftButtonPressed = TRUE;
			if (MAPED_HandleLeftButton (e) != 0) return -1;
			break;
		case 2:						// Mittlerer Button
			break;
		case 3:						// Rechter Button
			MAPED_RightButtonPressed = TRUE;
			if (MAPED_HandleRightButton (e) != 0) return -1; 
			break;	
	
	} // switch

	return 0;
}

// Handle a button up event
int MAPED_MouseButtonUp (SDL_MouseButtonEvent *e) {
	switch (e->button) {
		case 1: 
			MAPED_LeftButtonPressed = FALSE;
			break;
		case 2:
			break;
		case 3:
			MAPED_RightButtonPressed = FALSE;
			break;	
	} // switch	
	
	return 0;
} // MouseButtonUp

// Handle left Mouse Button events
// detect is the click went into the menu or map area
int MAPED_HandleLeftButton (SDL_MouseButtonEvent *e) {
//	printf ("%i %i %i %i\n", MENU_Area.x, MENU_Area.y, MENU_Area.w, MENU_Area.h);
	int i;

	for (i=0; i<10; i++) {
		if ((e->x > MENU_Area[i].x)
	  		&& (e->y > MENU_Area[i].y)
			&& (e->x < MENU_Area[i].x + MENU_Area[i].w)
 			&& (e->y < MENU_Area[i].y + MENU_Area[i].h)) {
				MENU_HandleMenu (e);
				return 0;
		} // if
	} // for

	MENU_HandleMap	(e);
	return 0;
}

// Handle clicks in the menu area
// here the menu_area-array is not relevant because the
// menu array is adressed directly
int MENU_HandleMenu (SDL_MouseButtonEvent *e) {
	int i;
	int sel_menu;

	// detect the selected button via the position
	i = 0;
	sel_menu = -1;
	for (i=0; i< MENU_MENULEVEL * MENU_MENUENTRIES; i++) {
		if ((menu[i].visible == TRUE) 
			&& (e->x > menu[i].x) 
			&& (e->y > menu[i].y) 
			&& (e->x < menu[i].x + menu[i].tile->img->w) 
			&& (e->y < menu[i].y + menu[i].tile->img->h)) {
				sel_menu = menu[i].index;
				break;
		} // if
	} // for

	// has something valid been chosen ?
	if (sel_menu == -1)  return -1; 

	// if menu - level == 1 then 
	// open a window with all menus layer == 2
	// if menu - level == 2 then
	// select the brush
	
	if (menu[sel_menu].level == MENU_LEVEL_1) {
	//	VIDEO_OpenWindow (); 
		VIDEO_ChangeText (txt_ln1, menu[i].text, 15, 25);
		MAPED_ShowMenuButtons (MENU_LEVEL_2);
		return 0;
	} else if (menu[sel_menu].level == MENU_LEVEL_2 && menu[sel_menu].visible == TRUE) {	
		// Draw a frame and memorize the selccted menu
		VIDEO_ChangeText (txt_ln1, menu[i].text, 15, 25);
		VIDEO_ShowObjectPos (frame_index, menu[sel_menu].x -2, menu[sel_menu].y-2); 
		MENU_Selected_Index = sel_menu;
		VIDEO_UpdateAll = TRUE;	
		return 0;
	} else 	
		return -1;

}

// Handle clicks on the map
int MENU_HandleMap	(SDL_MouseButtonEvent *e) {
	int map_index;
	struct TTile *brush;
	unsigned char zuf;
	
	if (MENU_Selected_Index == -1) return 0;
		
	map_index = MAP_GetTileforPos (e->x, e->y);
	if (map_index == -1) return 0;
			
	// gras:
	// get 1 random gras type
	if ((menu[MENU_Selected_Index].tile->type >= MT_GRAS_1) && (menu[MENU_Selected_Index].tile->type <= MT_GRAS_4)) {
		zuf = TOOLS_Random();
		if (zuf >0 && zuf <=64)  {
	      	brush = (struct TTile *) MAP_GetTileByType (MT_GRAS_1);
		} else if (zuf > 64 && zuf < 100) {
		    brush = (struct TTile *) MAP_GetTileByType (MT_GRAS_2);
		} else if (zuf > 100 && zuf < 200) {
      		brush = (struct TTile *) MAP_GetTileByType (MT_GRAS_3);
		} else {
      		brush = (struct TTile *) MAP_GetTileByType (MT_GRAS_4);
		}
	// trees
	} else if ((menu[MENU_Selected_Index].tile->type >= MT_TREE_1) && (menu[MENU_Selected_Index].tile->type <= MT_TREE_3)) {
		zuf = TOOLS_Random();
		if (zuf >0 && zuf <=80)  {
	      	brush = (struct TTile *) MAP_GetTileByType (MT_TREE_1);
		} else if (zuf > 80 && zuf < 160) {
		    brush = (struct TTile *) MAP_GetTileByType (MT_TREE_2);
		} else 
      		brush = (struct TTile *) MAP_GetTileByType (MT_TREE_3);
	// Special case: ways
	} else if ((menu[MENU_Selected_Index].tile->type >= MT_WAY_00) && (menu[MENU_Selected_Index].tile->type <= MT_WAY_15)) {
		brush = (struct TTile *) MAP_CreatePath (map_index);				
	} else
		brush = (struct TTile *) MAP_GetTileByType (menu[MENU_Selected_Index].brush->type);


	// set the tile
	MAP_SetTileIndex (map_index, brush->index, TRUE);
			
	VIDEO_UpdateMap = TRUE;

	return 0;
} // MENU_HandleMap


// Events Right Mousebutton
int MAPED_HandleRightButton (SDL_MouseButtonEvent *e) {
	return 0;
} // HandleRightButton


// Motion Events, triggerd by Right Mouse button and mouse movement:
// Handles Move-On-Motion: Move the map 
// and Draw-on-Mouse: Draw a brush wihle moving over the map
int MAPED_MouseMotion (SDL_MouseMotionEvent *e) {
		
	// check is any motion events are activated
	if ((MOVE_ON_MOUSE == 0) && (PAINT_ON_MOUSE == 0)) return 0;
	
	// move map via mouse 
	if (MOVE_ON_MOUSE && MAPED_RightButtonPressed) {	
		if (e->xrel > 0) MAP_MoveRight(1);
		if (e->xrel < 0) MAP_MoveLeft(1);		
		if (e->yrel > 0) MAP_MoveDown(1);
		if (e->yrel < 0) MAP_MoveUp(1);		
		VIDEO_UpdateMap = TRUE;
		return 0;
	}

	// draw per Motion
	// do it when activated by flag, the left mouse button is pressed and a brush is selected
	
	if (PAINT_ON_MOUSE == FALSE) return -1;
	if (e->state == SDL_RELEASED) return -1;
	if (MAPED_LeftButtonPressed == FALSE) return -1;
	return MAPED_HandleLeftButton (e);
	
} // MouseMotion
