/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


#include "include/video.h"

// initialize anything (SDE, Fonts, SDL, )
int VIDEO_InitSDL () {
	int i;
	
	// Initialize SDL
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0) return -1;
	atexit (SDL_Quit);
	
	// Get Screen Object
	if ((scr = SDL_SetVideoMode(SCR_X, SCR_Y, SCR_D, SDL_HWSURFACE)) < 0) return -1;
		
	// Set Screen Caption
	SDL_WM_SetCaption(TITLE, ICON);
	
	// Set Fullscreen	
	if (FULLSCREEN)	SDL_WM_ToggleFullScreen(scr);
	
	// Grab Mouse and Keyboard
	// SDL_WM_GrabInput(SDL_GRAB_ON);
	SDL_EnableKeyRepeat (SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL); 
	
	// Initialize the object array
	for (i=0; i<VIDEO_MAXOBJ; i++) 
		obj[i] = NULL;
	
	// init SGE 
	if (sge_TTF_Init () < 0) {
	    fprintf(stderr, "Kann SGE nicht initialisieren: %s\n", SDL_GetError());
        exit(1);		
	}
		
	// laod Fonts
	if ((font = sge_TTF_OpenFont ("/usr/share/fonts/truetype/freefont/FreeSans.ttf", 12)) == NULL) {
		fprintf(stderr, "Kann TrueTypeFont nicht initialisieren: %s\n", SDL_GetError());
        exit(1);
	}
	
	VIDEO_UpdateAll = TRUE;
	VIDEO_UpdateMap = TRUE;
	
	return 0;
}

//
// * Video Thread: 
//   Show all gfx
//   is triggerd by UpdateAll and UpdateMap bool !
// 
// the additional task of updating "All" when updating the map
// is done because of the fancy overlaying menu-frame.
// if the screen is divided strictly in 2 parts this can be disabled
int VIDEO_Thread () {
	while (1!=0) {
		if (VIDEO_UpdateMap == TRUE ) {
			MAP_ShowMap (MAP_posx, MAP_posy);
			VIDEO_UpdateMap = FALSE;
			VIDEO_DrawAll();					// additional
			SDL_Flip(scr); 				
			continue;
		}
	
		if (VIDEO_UpdateAll == TRUE ) {
			VIDEO_DrawAll();
			VIDEO_UpdateAll = FALSE;
			SDL_Flip(scr); 				
			continue;		
		}
		
		// give time to other processes
		SDL_Delay (1);
	} // While 1=0
	return 0;
} // VIDEO_Thread

int VIDEO_StartThread () {
	return (int) (video = SDL_CreateThread (VIDEO_Thread, NULL));	
}

void VIDEO_StopThread () {
	if (video != NULL) SDL_KillThread (video);
}


//
//
//  "object" interface
//
//


// Draw all objects from a list a the given order
void VIDEO_DrawAll () {

	// zuerst die MAP - Gras
	// dann alle Objekte auf der MAP (Bume, Viecher etc)
 	// dann der Hintergrundrahmen
	// dann alle Objekte auf dem Rahmen:
	// - Texte
	// - Icons
	// - irgendwelcheSachen
	
	// die Map zeichnet Showmap, wir konzentrieren uns hier auf den Rest
	// wir verwalten den Layerhandler und legen die Reihenfolge fest
	
	int i, layer;
	
	// fr jeden Layer, jedes Objekt lesen
	// sehr inperformat, da kein index per layer existiert
	// bei 5 layern und 100 objekten wird das hier schon 500 x durchlaufen

	for (layer=0; layer<VIDEO_MAXLAYER; layer++)
		for (i=0; i<VIDEO_MAXOBJ; i++)  		// lese jedes Objekt
			if (obj[i] != NULL) {				// ist es nicht null
				if (obj[i]->layer == layer) VIDEO_DrawObject (i); // und im richtigen layer
			}

} // VIDEO_DrawAll

// Create a object in the memory
// <- Pointer to new structure
struct TObject * VIDEO_CreateObject () {
	struct TObject *zeiger;
	
	zeiger = (struct TObject *) malloc (sizeof (struct TObject));
		
	return zeiger;	
}

// register a object into the array
// -> pointer to object to register
// <- index in array
int VIDEO_RegisterObject (struct TObject *o) {
	int i;
	
	for (i=0; i<VIDEO_MAXOBJ; i++) 
		if (obj[i] == NULL) { obj[i] = o; return i; }
			
	return -1;	
}

// Delete object from array and frees the memory
// -> index
void VIDEO_UnRegisterObject (int index) {
	VIDEO_FreeObject (obj[index]);
	obj[index] = NULL;
}

// Free memory
// -> pointer to memory to free
void VIDEO_FreeObject (struct TObject *o) {
	free (o);
}


// Draw a object depending on the "type"
// -> index
// <- 0 for success, -1 for error
int VIDEO_DrawObject (int index) {

	if (obj[index]->visible == V_INVISIBLE) 
		return 0;
	
	if (obj[index]->type == V_NONE) 
		return 0;
	
	if (obj[index]->type == V_SURFACE) {
		SDL_BlitSurface (obj[index]->data.surface, NULL, scr, &obj[index]->pos);
		return 0;
	}	
	
	if (obj[index]->type == V_TEXT) {
		// surface you draw on
		// font//	MENU_Show ();
		// forground RGB
		// background RGB
		// value for transparency
		// text (printf style possible) !
		sge_tt_textoutf (scr, font, 
		  obj[index]->pos.x, obj[index]->pos.y, 
		  255,255,30,
		  158,158,158,
		  255, obj[index]->data.text);		
		return 0;
	}

	if (obj[index]->type == V_INPUT) {
		// surface to draw on
		// font
		// string
		// Flags
		// pos = 0 for default text
		// len, maxchars editable
		// x, y pos
		// r,g,b forground
		// r,g,b background
		// alpha

		sge_tt_input (scr, font, obj[index]->data.text,
	                 SGE_IBG, 0, obj[index]->maxchars, 
					 obj[index]->pos.x, obj[index]->pos.y,
	                 0xFFFF30,
	                 0x585858, 100);
	
		return 0;
	}

	return -1;	
}


// Create a surface object (gfx)
// -> layer
// -> visisible TRUE / FALSE
// -> Pointer to the SDL_Surface to display
// -> Position as SDL_Rect Struct
// <- Pointer to the created object
struct TObject *VIDEO_CreateSurfaceObject ( int layer, int visible, SDL_Surface *surface, SDL_Rect *pos) {
	
	struct TObject *object;
	
	object = VIDEO_CreateObject ();
	
	object->type = V_SURFACE;
	object->layer = layer;
	object->visible = visible;
	object->data.surface = surface;
	
	if (pos == NULL) {
	 	object->pos.x = 0;
		object->pos.y = 0;
		object->pos.h = 0;
		object->pos.w = 0;
	} else {		
		object->pos.x = pos->x;
		object->pos.y = pos->y;
		object->pos.w = pos->w;
		object->pos.h = pos->h;		
	}
	return object;
}								

// Create a text object
// -> layer
// -> visisible TRUE / FALSE
// -> Pointer to text to display
// -> Position as SDL_Rect Struct
// <- Pointer to the created object
struct TObject * VIDEO_CreateTextObject (int layer, int visible, char *text, SDL_Rect *pos) {	
	struct TObject *object;
	
	object = VIDEO_CreateObject ();
	
	object->type = V_TEXT;
	object->layer = layer;
	object->visible = visible;
	object->data.text = text;
	if (pos == NULL) {
	 	object->pos.x = 0;
		object->pos.y = 0;
		object->pos.h = 0;
		object->pos.w = 0;
	} else {		
		object->pos.x = pos->x;
		object->pos.y = pos->y;
		object->pos.w = pos->w;
		object->pos.h = pos->h;		
	}

	return object;
}

// Creates and Prints out a text on x,y position
// returns index of this text
int VIDEO_TextOut (char *text, int x, int y) {
	SDL_Rect pos;
	struct TObject *p_obj;
	int index;
	
	pos.x = x;
	pos.y = y;
	p_obj = VIDEO_CreateTextObject(4, V_VISIBLE, text, &pos);
	index = VIDEO_RegisterObject (p_obj);
	return index;
}

// A Inputfield is like a Textfield only of a different type
// -> text (can be given, will be shown)
// -> textlen
// -> position x
// -> position y
// <- index of created object
// NOTE:
// input fields causes crashes when done in video thread !
// DO NOT USES THIS FUNCTION
// maybe when i find out why it crahses it can be fixed
int VIDEO_InputField (char *txt, int txtlen, int x, int y) {
	SDL_Rect pos;
	struct TObject *p_obj;
	int index;
	
	pos.x = x;
	pos.y = y;
	p_obj = VIDEO_CreateTextObject(4, V_VISIBLE, txt, &pos);
	p_obj->type = V_INPUT;
	p_obj->maxchars = txtlen;
	
	index = VIDEO_RegisterObject (p_obj);
	
	return index;	
}

// Hides a object
void VIDEO_HideObject (int index) {
	obj[index]->visible = V_INVISIBLE;
}

// Shows a object
void VIDEO_ShowObject (int index) {
	obj[index]->visible = V_VISIBLE;
}

// Shows a object on a new position
void VIDEO_ShowObjectPos (int index, int x, int y) {	
	obj[index]->visible = V_VISIBLE;
	obj[index]->pos.x = x;
	obj[index]->pos.y = y;
}

// change text and position of a text obj
void VIDEO_ChangeText (int index, char *new_text, int x, int y) {
	obj[index]->data.text = new_text;
	obj[index]->pos.x = x;
	obj[index]->pos.y = y;
}



// basic event function

int VIDEO_ShowMonthYear (char *str) {
	printf ("New month:%s\n", str);
	VIDEO_TextOut (str, 650, 25);
	VIDEO_UpdateAll = TRUE;
	return 0;
}
